//
//  MRBrew.h
//  MRBrew
//
//  Copyright (c) 2013 Marc Ransome <marc.ransome@fidgetbox.co.uk>
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to
//  deal in the Software without restriction, including without limitation the
//  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
//  sell copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//  DEALINGS IN THE SOFTWARE.
//

#import <Cocoa/Cocoa.h>
#import "MRBrewOperation.h"

enum {
    MRBrewErrorUnknown = 1
};

@protocol MRBrewDelegate;
@class MRBrewWorker;

/** The `MRBrew` class manages the execution of Homebrew operations. Operation
 * objects (defined by the MRBrewOperation class) are added to a queue and
 * remain there until they are explicitly cancelled or finish executing.
 *
 * Each call to performOperation:delegate: places an operation into the queue.
 * When an operation starts executing it will spawn a subprocess in a separate
 * thread. Multiple operations can be performed by making repeated
 * calls to performOperation:delegate:.
 *
 * By default, operations are executed concurrently, but this behaviour can be
 * controlled by setting concurrentOperations: to either `YES` or `NO`.
 *
 * `MRBrew`'s delegate methods—defined by the MRBrewDelegate protocol—allow
 * an object to receive callbacks regarding the success or failure of an
 * operation and output from Homebrew as it occurs.
 *
 * @warning Attempting to perform two operations that reference the same formula
 * concurrently may result in the failure of one of those operations. This is
 * the default behaviour for Homebrew.
 */
@interface MRBrew : NSObject

/**-----------------------------------------------------------------------------
 * @name Modifying the Homebrew path
 * -----------------------------------------------------------------------------
 */

/** Returns the absolute path of the Homebrew executable. */
+ (NSString *)brewPath;

/** Sets the absolute path of the Homebrew executable.
 *
 * @param path The absolute path of the Homebrew executable. If `nil` the
 * default path `/usr/local/bin/brew` will be used.
 */
+ (void)setBrewPath:(NSString *)path;

/**-----------------------------------------------------------------------------
 * @name Performing an Operation
 * -----------------------------------------------------------------------------
 */

/** Performs an operation.
 *
 * Operations are placed in a queue for execution and will always execute on
 * separate threads. Use concurrentOperations: to control how queued operations
 * are executed (i.e. concurrently, or serially).
 *
 * @param operation The operation to perform.
 * @param delegate The delegate object for the operation. The delegate will
 * receive delegate messages during execution of the operation when output is
 * generated and upon completion or failure of the operation.
 */
+ (void)performOperation:(MRBrewOperation *)operation delegate:(id<MRBrewDelegate>)delegate;

/**-----------------------------------------------------------------------------
 * @name Stopping an Operation
 * -----------------------------------------------------------------------------
 */

/** Cancels all queued and executing operations.
 *
 * This method has no effect if there are currently no queued operations.
 */
+ (void)cancelAllOperations;

/** Cancels a queued or executing operation.
 *
 * This method has no effect if the operation has already finished executing.
 *
 * @param operation The operation to cancel.
 */
+ (void)cancelOperation:(MRBrewOperation *)operation;

/** Cancels all queued and executing operations of the specified type.
 *
 * This method has no effect if there are currently no queued or executing
 * operations of the specified type.
 *
 * @param type The type of operations to cancel.
 */
+ (void)cancelAllOperationsOfType:(MRBrewOperationType)type;

/**-----------------------------------------------------------------------------
 * @name Managing Operations
 * -----------------------------------------------------------------------------
 */

/** Sets the concurrent execution of operations.
 *
 * By default, operations are executed concurrently. Changing concurrency type
 * does not affect operations that are currently executing. Operations are
 * always executed in separate threads.
 *
 * @param concurrency If `YES`, operations are executed concurrently. If `NO`,
 * operations are executed serially.
 */
+ (void)concurrentOperations:(BOOL)concurrency;

/** Returns the number of operations queued for execution.
 *
 * The value returned by this method represents the number of operation objects
 * queued for execution and will change as operations are completed.
 */
+ (NSUInteger)operationCount;

@end
